module Kululaskusilppuri where

import Data.ByteString.Lazy (ByteString)
import Data.List.NonEmpty (NonEmpty)
import Data.Vector (Vector)
import Data.Ord (comparing)
import System.IO (stdin)
import Control.Monad (forM_, when)
import Options.Applicative ((<**>))

import qualified Data.ByteString.Lazy as LBS
import qualified Data.Vector as Vector
import qualified Data.List as List
import qualified Data.List.NonEmpty as NE
import Data.Text.Lazy.Encoding as LTE
import Data.Text.Lazy as LT
import qualified Options.Applicative as O

import Data.Csv

import Kululaskusilppuri.Types

main :: IO ()
main = do
    outputPrefix <- O.execParser $ O.info ((O.argument O.str (O.metavar "OUTPUT-PREFIX")) <**> O.helper)
        (O.fullDesc <> O.progDesc "Muunna kululaskut.fi-CSV Kitsas-yhteensopivaksi"
                    <> O.header "kululaskusilppuri")
    kululaskut <- either error id . readKululaskut <$> LBS.hGetContents stdin
    when (Vector.null kululaskut) $ error "ei kululaskuja muokattavaksi!"
    let groupedKululaskut = NE.fromList $ NE.fromList <$> splitByDate kululaskut
        -- NE.fromList is safe here. groupBy only produces empty lists from
        -- empty lists, which are already guarded against by when ... error
    forM_ (writeKululaskut <$> groupedKululaskut) $ \(date, csv) ->
        LBS.writeFile (outputPrefix <> date <> ".csv") csv

readKululaskut :: ByteString -> Either String (Vector Kululasku)
readKululaskut = fmap (Vector.filter isPaid . snd) . decodeByName
        where isPaid = not . LBS.null . paymentDate

writeKululaskut :: NonEmpty Kululasku -> (String, ByteString)
writeKululaskut kululaskut =
    ( LT.unpack . LTE.decodeUtf8 . paymentDate $ NE.head kululaskut
    , encodeDefaultOrderedByName $ NE.toList kululaskut
    )

splitByDate :: Vector Kululasku -> [[Kululasku]]
splitByDate = List.groupBy (equal paymentDate) . List.sortBy (comparing paymentDate) . Vector.toList
    where equal f a b = f a == f b
